<!doctype html>
<html>
<head>
    <title>{{ name }}</title>
    <link rel="stylesheet" href="{{ url_for('static', filename='css/foundation.min.css') }}" />
    <link rel="stylesheet" href="{{ url_for('static', filename='icons/foundation-icons.css') }}" />
    <link rel="stylesheet" href="{{ url_for('static', filename='css/app.css') }}" />
    <script src="{{ url_for('static', filename='js/vendor/what-input.js') }}"></script>

    <script src="{{ url_for('static', filename='js/vendor/jquery.js') }}"></script>
    <script src="{{ url_for('static', filename='js/vendor/foundation.min.js') }}"></script>
    <script src="{{ url_for('static', filename='js/oboe-browser.min.js') }}"></script>

</head>
<body>
<div class=page>
    <div id="main-content">
        <div class="columns large-4 medium-6 small-12" id="arguments">
            <div class="top-bar" id="header">
                <div class="top-bar-title"><strong>{{ name }} - {{ description }}</strong></div>
                <div class="top-bar-right">
                    <div id="control-buttons">
                        <div class="button-group">
                            <button type="button" class="button success" id="sendButton" onclick="sendData()"> <i class="fi-play"></i> </button>
                            <button type="button" style="display: none;" class="button secondary disabled" disabled id="pauseButton" onclick="pauseProcess()"> <i class="fi-pause"></i> </button>
                            <button type="button" style="display: none;" class="button secondary" id="resumeButton" onclick="resumeProcess()"> <i class="fi-play"></i> </button>
                            <button type="button" class="button alert disabled" disabled id="stopButton" onclick="stopProcess()"> <i class="fi-stop"></i> </button>
                            <button type="button" class="button disabled" disabled id="reloadButton" onclick="reloadProcess()"> <i class="fi-refresh"></i> </button>
                            <button type="button" class="button" id="downloadButton" onclick="window.open('/download')"> <i class="fi-download"></i> </button>
                            <button type="button" class="button alert" id="clear-button" onclick="clearOutput()"> <i class="fi-x"></i></button>
                        </div>
                    </div>
                </div>
            </div>
            <ul id="actions" class="vertical menu">
            </ul>
        </div>
        <div class="columns large-8 medium-6 small-12" id="output_wrap">
            <div id="output"></div>
        </div>
    </div>
</div>
<script src="{{ url_for('static', filename='js/app.js') }}"></script>
</body>
</html>
